@@ -1,28 +1,36 @@
-from typing import Set, Optional, Dict, Protocol, Callable, TypeVar, Any, ClassVar, Tuple, cast
+from typing import Set, Optional, Dict, Callable, TypeVar, Any, ClassVar, Tuple
 from dataclasses import dataclass
 from abc import ABC, abstractmethod
-from functools import wraps
 
 from starlette.types import ASGIApp
 from starlette.requests import Request
 from starlette.responses import Response
 from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
 
-from .errors import Unauthorised, Forbidden
+from .errors import Unauthorised
 from .metrics import MetricsRegistry
 
+_REQUIRES_PRIVILEGE_ATTR = '_requires_privilege'
+
 @dataclass
 class Principal:
 	id: str
 	privileges: Set[str]
 
-class AuthBackend(Protocol):
+class AuthBackend(ABC):
+	@abstractmethod
 	async def resolve_principal(self, req: Request) -> Optional[Principal]:
 		...
 
+	@abstractmethod
 	def openapi_spec(self) -> Tuple[str, Dict[str, Any]]:
 		...
 
+	def principal_metric_labels(self, principal: Principal) -> Dict[str, str]:
+		return {
+			'principal': principal.id,
+		}
+
 class AuthMiddleware(BaseHTTPMiddleware):
 	def __init__(self, app: ASGIApp, auth_backend: AuthBackend, metrics_registry: MetricsRegistry) -> None:
 		super().__init__(app)
@@ -34,12 +42,14 @@ class AuthMiddleware(BaseHTTPMiddleware):
 			principal = await self.auth_backend.resolve_principal(req)
 
 		if principal is None:
-			return Unauthorised('authentication failed').as_response()
+			raise Unauthorised('authentication failed')
 
 		req.state.principal = principal
+		req.state.principal_labels = self.auth_backend.principal_metric_labels(principal)
+
 		return await call_next(req)
 
-class HeaderAuthBackend(ABC):
+class HeaderAuthBackend(AuthBackend):
 	header: ClassVar[str]
 
 	async def resolve_principal(self, req: Request) -> Optional[Principal]:
@@ -64,13 +74,6 @@ TFunc = TypeVar('TFunc', bound = Callable[..., Any])
 
 def privileged(privilege: str) -> Callable[[ TFunc ], TFunc]:
 	def _decorator(func: TFunc) -> TFunc:
-		@wraps(func)
-		async def _impl(*args: Any, **kwargs: Any) -> Any:
-			principal = cast(Principal, kwargs['principal'])
-			if privilege not in principal.privileges:
-				raise Forbidden('insufficient privileges')
-
-			return await func(*args, **kwargs)
-
-		return cast(TFunc, _impl)
+		setattr(func, _REQUIRES_PRIVILEGE_ATTR, privilege)
+		return func
 	return _decorator
