@@ -1,25 +1,21 @@
-from typing import Any, Optional, ClassVar
+import traceback
 from dataclasses import dataclass
 from http import HTTPStatus
-import traceback
+from typing import Any, ClassVar, Optional
 
-from starlette.requests import Request
-from starlette.responses import Response, JSONResponse
-from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
 from starlette.exceptions import HTTPException
+from starlette.middleware.base import (BaseHTTPMiddleware,
+                                       RequestResponseEndpoint)
+from starlette.requests import Request
+from starlette.responses import JSONResponse, Response
+
 
 @dataclass
-class HTTPError(HTTPException):
+class HTTPError(Exception):
 	desc: str
 	data: Optional[Any] = None
 	code: ClassVar[int]
 
-	def __post_init__(self) -> None:
-		super().__init__(self.code, self.desc)
-
-	def as_response(self) -> JSONResponse:
-		return error_response(self.code, self.desc, self.data)
-
 @dataclass
 class BadRequest(HTTPError):
 	code = 400
@@ -57,12 +53,12 @@ def error_response(code: int, desc: Optional[str], data: Optional[Any]) -> JSONR
 
 class JSONErrorMiddleware(BaseHTTPMiddleware):
 	async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
-		# TODO: logging
 		try:
 			response = await call_next(request)
 			return response
-		except HTTPError as e:
-			return e.as_response()
-		except Exception as e:
+		except HTTPError as ex:
+			return error_response(ex.code, ex.desc, ex.data)
+		except Exception as ex:
+			print("Internal Error")
 			traceback.print_exc()
 			return error_response(500, None, None)
